@@ -35,6 +35,7 @@
 
 #define WANTSOCKET
 #define WANTARPA
+#define WANTDNS
 #include "inet.h"
 
 #ifdef WIN32
@@ -515,7 +516,7 @@ dcc_chat_line (struct DCC *dcc, char *line, char *tbuf)
 		inbound_action (sess, dcc->serv->nick, dcc->nick, line + 8, FALSE);
 	} else
 	{
-		inbound_privmsg (dcc->serv, dcc->nick, "", line);
+		inbound_privmsg (dcc->serv, dcc->nick, "", line, FALSE);
 	}
 	if (utf)
 		g_free (utf);
@@ -611,13 +612,21 @@ dcc_calc_average_cps (struct DCC *dcc)
 		dcc->cps = (dcc->pos - dcc->resumable) / sec;
 }
 
+static void
+dcc_send_ack (struct DCC *dcc)
+{
+	/* send in 32-bit big endian */
+	guint32 pos = htonl (dcc->pos);
+	send (dcc->sok, (char *) &pos, 4, 0);
+}
+
 static gboolean
 dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 {
 	char *old;
 	char buf[4096];
-	guint32 pos;
 	int n;
+	gboolean need_ack = FALSE;
 
 	if (dcc->fp == -1)
 	{
@@ -669,6 +678,9 @@ dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	{
 		if (dcc->throttled)
 		{
+			if (need_ack)
+				dcc_send_ack (dcc);
+
 			fe_input_remove (dcc->iotag);
 			dcc->iotag = 0;
 			return FALSE;
@@ -683,11 +695,18 @@ dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 			if (n < 0)
 			{
 				if (would_block_again ())
+				{
+					if (need_ack)
+						dcc_send_ack (dcc);
 					return TRUE;
+				}
 			}
 			EMIT_SIGNAL (XP_TE_DCCRECVERR, dcc->serv->front_session, dcc->file,
 							 dcc->destfile, dcc->nick,
 							 errorstring ((n < 0) ? sock_error () : 0), 0);
+			/* send ack here? but the socket is dead */
+			/*if (need_ack)
+				dcc_send_ack (dcc);*/
 			dcc_close (dcc, STAT_FAILED, FALSE);
 			return TRUE;
 		}
@@ -696,17 +715,19 @@ dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 		{
 			EMIT_SIGNAL (XP_TE_DCCRECVERR, dcc->serv->front_session, dcc->file,
 							 dcc->destfile, dcc->nick, errorstring (errno), 0);
+			if (need_ack)
+				dcc_send_ack (dcc);
 			dcc_close (dcc, STAT_FAILED, FALSE);
 			return TRUE;
 		}
-		dcc->pos += n;
-		pos = htonl (dcc->pos);
-		send (dcc->sok, (char *) &pos, 4, 0);
 
 		dcc->lasttime = time (0);
+		dcc->pos += n;
+		need_ack = TRUE;	/* send ack when we're done recv()ing */
 
 		if (dcc->pos >= dcc->size)
 		{
+			dcc_send_ack (dcc);
 			dcc_calc_average_cps (dcc);
 			sprintf (buf, "%d", dcc->cps);
 			dcc_close (dcc, STAT_DONE, FALSE);
@@ -714,8 +735,6 @@ dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 							 dcc->file, dcc->destfile, dcc->nick, buf, 0);
 			return TRUE;
 		}
-
-		dcc_calc_cps (dcc);
 	}
 }
 
@@ -907,7 +926,6 @@ abortit:
 	{
 		dcc->pos += sent;
 		dcc->lasttime = time (0);
-		dcc_calc_cps (dcc);
 	}
 
 	/* have we sent it all yet? */
@@ -1045,6 +1063,7 @@ dcc_listen_init (struct DCC *dcc, session *sess)
 {
 	unsigned long my_addr;
 	struct sockaddr_in SAddr;
+	struct hostent *dns_query;
 	int i, bindretval = -1;
 	socklen_t len;
 
@@ -1082,11 +1101,14 @@ dcc_listen_init (struct DCC *dcc, session *sess)
 			/*printf("Trying to bind against port: %d\n",ntohs(SAddr.sin_port));*/
 			bindretval = bind (dcc->sok, (struct sockaddr *) &SAddr, sizeof (SAddr));
 		}
-	}
 
-	/*if we didnt bind yet, try a random port*/
-	if (bindretval == -1)
+		/* with a small port range, reUseAddr is needed */
+		len = 1;
+		setsockopt (dcc->sok, SOL_SOCKET, SO_REUSEADDR, (char *) &len, sizeof (len));
+
+	} else
 	{
+		/* try random port */
 		SAddr.sin_port = 0;
 		bindretval = bind (dcc->sok, (struct sockaddr *) &SAddr, sizeof (SAddr));
 	}
@@ -1104,14 +1126,30 @@ dcc_listen_init (struct DCC *dcc, session *sess)
 	dcc->port = ntohs (SAddr.sin_port);
 
 	/*if we have a dcc_ip, we use that, so the remote client can connect*/
-	/*we would tell the client to connect to our LAN ip otherwise*/
+	/*else we try to take an address from dcc_ip_str*/
+	/*if something goes wrong we tell the client to connect to our LAN ip*/
+
+	dcc->addr = 0;
+
 	if (prefs.ip_from_server != 0 && prefs.dcc_ip != 0)
 		dcc->addr = prefs.dcc_ip;
 	else if (prefs.dcc_ip_str[0])
-		dcc->addr = inet_addr ((const char *) prefs.dcc_ip_str);
-	else
-	/*else we use the one we bound to*/
-		dcc->addr = my_addr;
+	{
+	   dns_query = gethostbyname ((const char *) prefs.dcc_ip_str);
+
+	   if (dns_query != NULL &&
+	       dns_query->h_length == 4 &&
+	       dns_query->h_addr_list[0] != NULL)
+		  {
+		    /*we're offered at least one IPv4 address: we take the first*/
+
+		    dcc->addr = (unsigned long) *((guint32*) dns_query->h_addr_list[0]);
+		  }
+	}
+
+	/*if nothing else worked we use the address we bound to*/
+	if (dcc->addr == 0)
+	   dcc->addr = my_addr;
 
 	dcc->addr = ntohl (dcc->addr);
 
@@ -1150,6 +1188,7 @@ dcc_send (struct session *sess, char *to, char *file, int maxcps, int passive)
 	{
 		char path[256];
 		char wild[256];
+		char *path_fs;	/* local filesystem encoding */
 
 		safe_strcpy (wild, file_part (file), sizeof (wild));
 		path_part (file, path, sizeof (path));
@@ -1162,9 +1201,15 @@ dcc_send (struct session *sess, char *to, char *file, int maxcps, int passive)
 
 		free (file);
 
-		recursive = TRUE;
-		for_files (path, wild, dcc_send_wild);
-		recursive = FALSE;
+		/* for_files() will use opendir, so we need local FS encoding */
+		path_fs = g_filename_from_utf8 (path, -1, 0, 0, 0);
+		if (path_fs)
+		{
+			recursive = TRUE;
+			for_files (path_fs, wild, dcc_send_wild);
+			recursive = FALSE;
+			g_free (path_fs);
+		}
 
 		return;
 	}
@@ -1190,7 +1235,7 @@ dcc_send (struct session *sess, char *to, char *file, int maxcps, int passive)
 		{
 			if (st.st_size > 0)
 			{
-				dcc->offertime = time (0);
+				dcc->starttime = dcc->offertime = time (0);
 				dcc->serv = sess->server;
 				dcc->dccstat = STAT_QUEUED;
 				dcc->size = st.st_size;
@@ -1594,7 +1639,7 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 	if (!strcasecmp (type, "CHAT"))
 	{
 		port = atoi (word[8]);
-		sscanf (word[7], "%lu", &addr);
+		addr = strtoul (word[7], NULL, 10);
 
 		if (port == 0)
 			pasvid = atoi (word[9]);
@@ -1662,7 +1707,7 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 			dcc = find_dcc (nick, word[6], TYPE_SEND);
 		if (dcc)
 		{
-			sscanf (word[8], "%lu", &size);
+			size = strtoul (word[8], NULL, 10);
 			dcc->resumable = size;
 			if (dcc->resumable < dcc->size)
 			{
@@ -1706,8 +1751,8 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 		int psend = 0;
 
 		port = atoi (word[8]);
-		sscanf (word[7], "%lu", &addr);
-		sscanf (word[9], "%lu", &size);
+		addr = strtoul (word[7], NULL, 10);
+		size = strtoul (word[9], NULL, 10);
 
 		if (port == 0) /* Passive dcc requested */
 			pasvid = atoi (word[10]);
