@@ -18,10 +18,6 @@
 
 #include <stdio.h>
 #include <string.h>
-#include "../../config.h"
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -32,8 +28,8 @@
 #include "xchat.h"
 #include "notify.h"
 #include "cfgfiles.h"
-#include "plugin.h"
 #include "fe.h"
+#include "server.h"
 #include "text.h"
 #include "util.h"
 #include "xchatc.h"
@@ -43,7 +39,7 @@ GSList *notify_list = 0;
 int notify_tag = 0;
 
 
-static struct notify_per_server *
+struct notify_per_server *
 notify_find_server_entry (struct notify *notify, struct server *serv)
 {
 	GSList *list = notify->server_list;
@@ -74,7 +70,7 @@ notify_save (void)
 	struct notify *notify;
 	GSList *list = notify_list;
 
-	snprintf (buf, sizeof buf, "%s/notify.conf", get_xdir ());
+	snprintf (buf, sizeof buf, "%s/notify.conf", get_xdir_fs ());
 
 	fh = open (buf, O_TRUNC | O_WRONLY | O_CREAT | OFLAGS, 0600);
 	if (fh != -1)
@@ -96,14 +92,14 @@ notify_load (void)
 	int fh;
 	char buf[256];
 
-	snprintf (buf, sizeof buf, "%s/notify.conf", get_xdir ());
+	snprintf (buf, sizeof buf, "%s/notify.conf", get_xdir_fs ());
 
 	fh = open (buf, O_RDONLY | OFLAGS);
 	if (fh != -1)
 	{
-		while (waitline (fh, buf, sizeof buf) != -1)
+		while (waitline (fh, buf, sizeof buf, FALSE) != -1)
 		{
-			if (*buf != '#')
+			if (buf[0] != '#' && buf[0] != 0)
 				notify_adduser (buf);
 		}
 		close (fh);
@@ -128,7 +124,7 @@ notify_find (server *serv, char *nick)
 			continue;
 		}
 
-		if (!strcasecmp (notify->name, nick))
+		if (!serv->p_cmp (notify->name, nick))
 			return servnot;
 
 		list = list->next;
@@ -144,8 +140,6 @@ notify_announce_offline (server * serv, struct notify_per_server *servnot,
 	session *sess;
 
 	sess = serv->front_session;
-	if (prefs.show_notify_in_front_session && menu_sess)
-		sess = menu_sess;
 
 	servnot->ison = FALSE;
 	servnot->lastoff = time (0);
@@ -160,12 +154,9 @@ static void
 notify_announce_online (server * serv, struct notify_per_server *servnot,
 								char *nick)
 {
-	char buf[256];
 	session *sess;
 
 	sess = serv->front_session;
-	if (prefs.show_notify_in_front_session && menu_sess)
-		sess = menu_sess;
 
 	servnot->lastseen = time (0);
 	if (servnot->ison)
@@ -180,8 +171,13 @@ notify_announce_online (server * serv, struct notify_per_server *servnot,
 
 	if (prefs.whois_on_notifyonline)
 	{
-		snprintf (buf, sizeof (buf), "WHOIS %s\r\n", nick);
-		tcp_send (serv, buf);
+
+	    /* Let's do whois with idle time (like in /quote WHOIS %s %s) */
+
+	    char *wii_str = malloc (strlen (nick) * 2 + 2);
+	    sprintf (wii_str, "%s %s", nick, nick);
+	    serv->p_whois (serv, wii_str);
+	    free (wii_str);
 	}
 }
 
@@ -218,10 +214,10 @@ notify_watch (server * serv, char *nick, int add)
 {
 	char tbuf[256];
 
-	snprintf (tbuf, sizeof (tbuf), "WATCH +%s\r\n", nick);
+	snprintf (tbuf, sizeof (tbuf), "WATCH +%s", nick);
 	if (!add)
 		tbuf[6] = '-';
-	tcp_send (serv, tbuf);
+	serv->p_raw (serv, tbuf);
 }
 
 static void
@@ -255,8 +251,7 @@ notify_flush_watches (server * serv, GSList *from, GSList *end)
 		strcat (tbuf, notify->name);
 		list = list->next;
 	}
-	strcat (tbuf, "\r\n");
-	tcp_send (serv, tbuf);
+	serv->p_raw (serv, tbuf);
 }
 
 /* called when logging in. e.g. when End of motd. */
@@ -269,9 +264,6 @@ notify_send_watches (server * serv)
 	GSList *point;
 	int len;
 
-	if (!serv)
-		return;
-
 	len = 0;
 	point = list = notify_list;
 	while (list)
@@ -283,7 +275,7 @@ notify_send_watches (server * serv)
 		if (len > 500)
 		{
 			notify_flush_watches (serv, point, list);
-			len = 0;
+			len = strlen (notify->name) + 2;
 			point = list;
 		}
 
@@ -317,7 +309,7 @@ notify_markonline (server *serv, char *word[])
 		seen = FALSE;
 		while (*word[i])
 		{
-			if (!strcasecmp (notify->name, word[i]))
+			if (!serv->p_cmp (notify->name, word[i]))
 			{
 				seen = TRUE;
 				notify_announce_online (serv, servnot, notify->name);
@@ -376,13 +368,12 @@ notify_checklist (void)
 	}
 	if (i)
 	{
-		GSList *list = serv_list;
-		strcat (outbuf, "\r\n");
+		list = serv_list;
 		while (list)
 		{
 			serv = (struct server *) list->data;
 			if (serv->connected && serv->end_of_motd && !serv->supports_watch)
-				tcp_send (serv, outbuf);
+				serv->p_raw (serv, outbuf);
 			list = list->next;
 		}
 	}
@@ -430,7 +421,7 @@ notify_deluser (char *name)
 	while (list)
 	{
 		notify = (struct notify *) list->data;
-		if (!strcasecmp (notify->name, name))
+		if (!rfc_casecmp (notify->name, name))
 		{
 			fe_notify_update (notify->name);
 			/* Remove the records for each server */
@@ -480,7 +471,7 @@ notify_isnotify (struct session *sess, char *name)
 	while (list)
 	{
 		notify = (struct notify *) list->data;
-		if (!strcasecmp (notify->name, name))
+		if (!sess->server->p_cmp (notify->name, name))
 		{
 			servnot = notify_find_server_entry (notify, sess->server);
 			if (servnot && servnot->ison)
@@ -529,6 +520,7 @@ notify_cleanup ()
 			{
 				notify->server_list =
 					g_slist_remove (notify->server_list, servnot);
+				free (servnot);
 				nslist = notify->server_list;
 			} else
 			{
