@@ -64,6 +64,22 @@ irc_login (server *serv, char *user, char *realname)
 				  serv->nick, user, hostname, serv->servername, realname);
 }
 
+static void
+irc_nickserv (server *serv, char *pass)
+{
+	switch (serv->nickservtype)
+	{
+	case 0:
+		tcp_sendf (serv, "PRIVMSG NickServ :identify %s\r\n", pass);
+		break;
+	case 1:
+		tcp_sendf (serv, "NICKSERV :identify %s\r\n", pass);
+		break;
+	case 2:
+		tcp_sendf (serv, "NS :identify %s\r\n", pass);
+	}
+}
+
 static void
 irc_join (server *serv, char *channel, char *key)
 {
@@ -137,15 +153,9 @@ irc_invite (server *serv, char *channel, char *nick)
 }
 
 static void
-irc_chan_mode (server *serv, char *channel, char *mode)
+irc_mode (server *serv, char *target, char *mode)
 {
-	tcp_sendf (serv, "MODE %s %s\r\n", channel, mode);
-}
-
-static void
-irc_nick_mode (server *serv, char *nick, char *mode)
-{
-	tcp_sendf (serv, "MODE %s %s\r\n", nick, mode);
+	tcp_sendf (serv, "MODE %s %s\r\n", target, mode);
 }
 
 /* find channel info when joined */
@@ -164,6 +174,14 @@ irc_user_list (server *serv, char *channel)
 	tcp_sendf (serv, "WHO %s\r\n", channel);
 }
 
+/* userhost */
+
+static void
+irc_userhost (server *serv, char *nick)
+{
+	tcp_sendf (serv, "USERHOST %s\r\n", nick);
+}
+
 static void
 irc_away_status (server *serv, char *channel)
 {
@@ -288,7 +306,7 @@ channel_date (session *sess, char *chan, char *timestr)
 {
 	time_t timestamp = (time_t) atol (timestr);
 	char *tim = ctime (&timestamp);
-	tim[19] = 0;	/* get rid of the \n */
+	tim[24] = 0;	/* get rid of the \n */
 	EMIT_SIGNAL (XP_TE_CHANDATE, sess, chan, tim, NULL, NULL, 0);
 }
 
@@ -308,12 +326,21 @@ process_numeric (session * sess, int n,
 		if ((strncmp(word[7], "PTnet", 5) == 0) &&
 			(strncmp(word[8], "IRC", 3) == 0) &&
 			(strncmp(word[9], "Network", 7) == 0) &&
-			(strchr(word[10], '@') != NULL))
+			(strrchr(word[10], '@') != NULL))
 		{
 			serv->use_who = FALSE;
 			if (prefs.ip_from_server)
-				inbound_foundip (sess, strchr(word[10], '@')+1);
+				inbound_foundip (sess, strrchr(word[10], '@')+1);
 		}
+
+		/* use /NICKSERV */
+		if (strcasecmp (word[7], "DALnet") == 0)
+			serv->nickservtype = 1;
+
+		/* use /NS */
+		else if (strcasecmp (word[7], "freenode") == 0)
+			serv->nickservtype = 2;
+
 		goto def;
 
 	case 4:	/* check the ircd type */
@@ -341,11 +368,38 @@ process_numeric (session * sess, int n,
 			fe_chan_list_end (sess->server);
 		goto def;
 
+	case 290:	/* CAPAB reply */
+		if (strstr (word_eol[1], "IDENTIFY-MSG"))
+		{
+			serv->have_idmsg = TRUE;
+			break;
+		}
+		goto def;
+
 	case 301:
 		inbound_away (serv, word[4],
 						(word_eol[5][0] == ':') ? word_eol[5] + 1 : word_eol[5]);
 		break;
 
+	case 302:
+		if (serv->skip_next_userhost)
+		{
+			char *eq = strchr (word[4], '=');
+			if (eq)
+			{
+				*eq = 0;
+				if (!serv->p_cmp (word[4] + 1, serv->nick))
+				{
+					char *at = strrchr (eq + 1, '@');
+					if (at)
+						inbound_foundip (sess, at + 1);
+				}
+			}
+
+			serv->skip_next_userhost = FALSE;
+		}
+		else goto def;
+
 	case 303:
 		word[4]++;
 		notify_markonline (serv, word);
@@ -416,7 +470,7 @@ process_numeric (session * sess, int n,
 
 	case 321:
 		if (!fe_is_chanwindow (sess->server))
-			EMIT_SIGNAL (XP_TE_CHANLISTHEAD, sess, NULL, NULL, NULL, NULL, 0);
+			EMIT_SIGNAL (XP_TE_CHANLISTHEAD, serv->server_session, NULL, NULL, NULL, NULL, 0);
 		break;
 
 	case 322:
@@ -425,14 +479,14 @@ process_numeric (session * sess, int n,
 			fe_add_chan_list (sess->server, word[4], word[5], word_eol[6] + 1);
 		} else
 		{
-			PrintTextf (sess, "%-16s %-7d %s\017\n",
+			PrintTextf (serv->server_session, "%-16s %-7d %s\017\n",
 							word[4], atoi (word[5]), word_eol[6] + 1);
 		}
 		break;
 
 	case 323:
 		if (!fe_is_chanwindow (sess->server))
-			EMIT_SIGNAL (XP_TE_SERVTEXT, sess, text, word[1], NULL, NULL, 0);
+			EMIT_SIGNAL (XP_TE_SERVTEXT, serv->server_session, text, word[1], NULL, NULL, 0);
 		else
 			fe_chan_list_end (sess->server);
 		break;
@@ -495,7 +549,6 @@ process_numeric (session * sess, int n,
 		break;
 
 	case 352:						  /* WHO */
-		if (!serv->skip_next_who)
 		{
 			unsigned int away = 0;
 			session *who_sess = find_channel (serv, word[4]);
@@ -510,10 +563,6 @@ process_numeric (session * sess, int n,
 			if (!who_sess || !who_sess->doing_who)
 				EMIT_SIGNAL (XP_TE_SERVTEXT, serv->server_session, text, word[1],
 								 NULL, NULL, 0);
-		} else
-		{
-			if (!serv->p_cmp (word[8], serv->nick))
-				inbound_foundip (sess, word[6]);
 		}
 		break;
 
@@ -543,10 +592,6 @@ process_numeric (session * sess, int n,
 		break;
 
 	case 315:						  /* END OF WHO */
-		if (serv->skip_next_who)
-		{
-			serv->skip_next_who = FALSE;
-		} else
 		{
 			session *who_sess;
 			who_sess = find_channel (serv, word[4]);
@@ -715,9 +760,24 @@ process_named_msg (session *sess, char *type, char *word[], char *word_eol[])
 	}
 	if (!strcmp ("NOTICE", type))
 	{
+		int id = FALSE;	/* identified */
+
+		text = word_eol[4];
+		if (*text == ':')
+			text++;
+
+		if (serv->have_idmsg)
+		{
+			if (*text == '+')
+			{
+				id = TRUE;
+				text++;
+			} else if (*text == '-')
+				text++;
+		}
+
 		if (!ignore_check (word[1], IG_NOTI))
-			inbound_notice (serv, word[3], nick,
-					(word_eol[4][0] == ':') ? word_eol[4] + 1 : word_eol[4], ip);
+			inbound_notice (serv, word[3], nick, text, ip, id);
 		return;
 	}
 	if (!strcmp ("PART", type))
@@ -739,11 +799,21 @@ process_named_msg (session *sess, char *type, char *word[], char *word_eol[])
 	{
 		char *to = word[3];
 		int len;
+		int id = FALSE;	/* identified */
 		if (*to)
 		{
 			text = word_eol[4];
 			if (*text == ':')
 				text++;
+			if (serv->have_idmsg)
+			{
+				if (*text == '+')
+				{
+					id = TRUE;
+					text++;
+				} else if (*text == '-')
+					text++;
+			}
 			len = strlen (text);
 			if (text[0] == 1 && text[len - 1] == 1)	/* ctcp */
 			{
@@ -761,12 +831,12 @@ process_named_msg (session *sess, char *type, char *word[], char *word_eol[])
 				{
 					if (ignore_check (word[1], IG_CHAN))
 						return;
-					inbound_chanmsg (serv, to, nick, text, FALSE);
+					inbound_chanmsg (serv, NULL, to, nick, text, FALSE, id);
 				} else
 				{
 					if (ignore_check (word[1], IG_PRIV))
 						return;
-					inbound_privmsg (serv, nick, ip, text);
+					inbound_privmsg (serv, nick, ip, text, id);
 				}
 			}
 		}
@@ -935,16 +1005,17 @@ proto_fill_her_up (server *serv)
 	serv->p_quit = irc_quit;
 	serv->p_kick = irc_kick;
 	serv->p_part = irc_part;
+	serv->p_nickserv = irc_nickserv;
 	serv->p_join = irc_join;
 	serv->p_login = irc_login;
 	serv->p_join_info = irc_join_info;
-	serv->p_chan_mode = irc_chan_mode;
-	serv->p_nick_mode = irc_nick_mode;
+	serv->p_mode = irc_mode;
 	serv->p_user_list = irc_user_list;
 	serv->p_away_status = irc_away_status;
 	/*serv->p_get_ip = irc_get_ip;*/
 	serv->p_whois = irc_user_whois;
 	serv->p_get_ip = irc_user_list;
+	serv->p_get_ip_uh = irc_userhost;
 	serv->p_set_back = irc_set_back;
 	serv->p_set_away = irc_set_away;
 	serv->p_message = irc_message;
