@@ -159,11 +159,18 @@ tcp_send_queue (server *serv)
 				if (serv->next_send < now)
 					serv->next_send = now;
 				if (serv->next_send - now >= 10)
-					return 1;				  /* don't remove the timeout handler */
+				{
+					/* check for clock skew */
+					if (now >= serv->prev_now)
+						return 1;		  /* don't remove the timeout handler */
+					/* it is skewed, reset to something sane */
+					serv->next_send = now;
+				}
 
 				for (p = buf, i = len; i && *p != ' '; p++, i--);
 				serv->next_send += (2 + i / 120);
 				serv->sendq_len -= len;
+				serv->prev_now = now;
 				fe_set_throttle (serv);
 
 				tcp_send_real (serv, buf, len);
@@ -433,24 +440,8 @@ server_read (GIOChannel *source, GIOCondition condition, server *serv)
 static void
 server_connected (server * serv)
 {
-	char outbuf[512];
-
-	serv->ping_recv = time (0);
-
-#ifdef WIN32
-	if (prefs.identd)
-		identd_start ();
-#else
-	sprintf (outbuf, "%s/auth/xchat_auth", g_get_home_dir ());
-	if (access (outbuf, X_OK) == 0)
-	{
-		sprintf (outbuf, "exec -d %s/auth/xchat_auth %s", g_get_home_dir (),
-					prefs.username);
-		handle_command (serv->server_session, outbuf, FALSE);
-	}
-#endif
-
 	prefs.wait_on_exit = TRUE;
+	serv->ping_recv = time (0);
 	serv->connected = TRUE;
 	set_nonblocking (serv->sok);
 	serv->iotag = fe_input_add (serv->sok, FIA_READ|FIA_EX, server_read, serv);
@@ -458,9 +449,19 @@ server_connected (server * serv)
 	{
 		EMIT_SIGNAL (XP_TE_CONNECTED, serv->server_session, NULL, NULL, NULL,
 						 NULL, 0);
-		serv->p_login (serv,
-							(serv->username) ? serv->username : prefs.username,
-							(serv->realname) ? serv->realname : prefs.realname);
+		if (serv->network)
+		{
+			serv->p_login (serv,
+								(((ircnet *)serv->network)->user) ?
+								(((ircnet *)serv->network)->user) :
+								prefs.username,
+								(((ircnet *)serv->network)->real) ?
+								(((ircnet *)serv->network)->real) :
+								prefs.realname);
+		} else
+		{
+			serv->p_login (serv, prefs.username, prefs.realname);
+		}
 	} else
 	{
 		EMIT_SIGNAL (XP_TE_SERVERCONNECTED, serv->server_session, NULL, NULL,
@@ -470,6 +471,19 @@ server_connected (server * serv)
 	set_server_name (serv, serv->servername);
 }
 
+#ifdef WIN32
+
+static gboolean
+server_close_pipe (int *pipefd)	/* see comments below */
+{
+	close (pipefd[0]);	/* close WRITE end first to cause an EOF on READ */
+	close (pipefd[1]);	/* in giowin32, and end that thread. */
+	free (pipefd);
+	return FALSE;
+}
+
+#endif
+
 static void
 server_stopconnecting (server * serv)
 {
@@ -483,12 +497,20 @@ server_stopconnecting (server * serv)
 	/* kill the child process trying to connect */
 	kill (serv->childpid, SIGKILL);
 	waitpid (serv->childpid, NULL, 0);
-#else
-	PostThreadMessage (serv->childpid, WM_QUIT, 0, 0);
-#endif
 
 	close (serv->childwrite);
 	close (serv->childread);
+#else
+	PostThreadMessage (serv->childpid, WM_QUIT, 0, 0);
+
+	{
+		/* if we close the pipe now, giowin32 will crash. */
+		int *pipefd = malloc (sizeof (int) * 2);
+		pipefd[0] = serv->childwrite;
+		pipefd[1] = serv->childread;
+		g_idle_add ((GSourceFunc)server_close_pipe, pipefd);
+	}
+#endif
 
 #ifdef USE_OPENSSL
 	if (serv->ssl_do_connect_tag)
@@ -560,6 +582,9 @@ ssl_do_connect (server * serv)
 			EMIT_SIGNAL (XP_TE_CONNFAIL, serv->server_session, buf, NULL,
 							 NULL, NULL, 0);
 
+			if (ERR_GET_REASON (err) == SSL_R_WRONG_VERSION_NUMBER)
+				PrintText (serv->server_session, _("Are you sure this is a SSL capable server and port?\n"));
+
 			server_cleanup (serv);
 
 			if (prefs.autoreconnectonfail)
@@ -780,7 +805,7 @@ waitline2 (GIOChannel *source, char *buf, int bufsize)
 
 #else
 
-#define waitline2(source,buf,size) waitline(serv->childread,buf,size)
+#define waitline2(source,buf,size) waitline(serv->childread,buf,size,0)
 
 #endif
 
@@ -835,6 +860,10 @@ server_read_child (GIOChannel *source, GIOCondition condition, server *serv)
 
 	switch (tbuf[0])
 	{
+	case '0':	/* print some text */
+		waitline2 (source, tbuf, sizeof tbuf);
+		PrintText (serv->server_session, tbuf);
+		break;
 	case '1':						  /* unknown host */
 		server_stopconnecting (serv);
 		closesocket (serv->sok4);
@@ -866,6 +895,26 @@ server_read_child (GIOChannel *source, GIOCondition condition, server *serv)
 		waitline2 (source, ip, sizeof ip);
 		waitline2 (source, outbuf, sizeof outbuf);
 		EMIT_SIGNAL (XP_TE_CONNECT, sess, host, ip, outbuf, NULL, 0);
+#ifdef WIN32
+		if (prefs.identd)
+		{
+			if (serv->network)
+				identd_start ((((ircnet *)serv->network)->user) ?
+									(((ircnet *)serv->network)->user) :
+									prefs.username);
+			else
+				identd_start (prefs.username);
+		}
+#else
+		snprintf (outbuf, sizeof (outbuf), "%s/auth/xchat_auth",
+					 g_get_home_dir ());
+		if (access (outbuf, X_OK) == 0)
+		{
+			snprintf (outbuf, sizeof (outbuf), "exec -d %s/auth/xchat_auth %s",
+						 g_get_home_dir (), prefs.username);
+			handle_command (serv->server_session, outbuf, FALSE);
+		}
+#endif
 		break;
 	case '4':						  /* success */
 		waitline2 (source, tbuf, sizeof (tbuf));
@@ -937,6 +986,7 @@ server_cleanup (server * serv)
 	{
 		close_socket (serv->sok);
 		serv->connected = FALSE;
+		serv->end_of_motd = FALSE;
 		return 2;
 	}
 
@@ -961,11 +1011,6 @@ server_disconnect (session * sess, int sendquit, int err)
 	/* send our QUIT reason */
 	if (sendquit && serv->connected)
 	{
-		if (serv->eom_cmd)
-		{
-			free (serv->eom_cmd);
-			serv->eom_cmd = NULL;
-		}
 		server_sendquit (sess);
 	}
 
@@ -1054,16 +1099,56 @@ traverse_socks5 (int sok, char *serverAddr, int port)
 	struct sock5_connect1 sc1;
 	unsigned char *sc2;
 	unsigned int packetlen, addrlen;
-	unsigned char buf[10];
+	unsigned char buf[260];
 
 	sc1.version = 5;
 	sc1.nmethods = 1;
-	sc1.method = 0;
+	if ( prefs.proxy_auth )
+	{
+		if ( !prefs.proxy_user[0] || !prefs.proxy_pass[0] )
+		{
+			return 1;
+		}
+		sc1.method = 2;  /* Username/Password Authentication (UPA) */
+	}
+	else
+	{
+		sc1.method = 0;  /* NO Authentication */
+	}
 	send (sok, (char *) &sc1, 3, 0);
 	if (recv (sok, buf, 2, 0) != 2)
 		return 1;
-	if (buf[0] != 5 && buf[1] != 0)
-		return 1;
+
+	if ( prefs.proxy_auth )
+	{
+		int len_u=0, len_p=0;
+
+		/* authentication sub-negotiation (RFC1929) */
+		if ( buf[0] != 5 || buf[1] != 2 )  /* UPA not supported by server */
+			return 1;
+
+		memset (buf, 0, sizeof(buf));
+
+		/* form the UPA request */
+		len_u = strlen (prefs.proxy_user);
+		len_p = strlen (prefs.proxy_pass);
+		buf[0] = 1;
+		buf[1] = len_u;
+		memcpy (buf + 2, prefs.proxy_user, len_u);
+		buf[2 + len_u] = len_p;
+		memcpy (buf + 3 + len_u, prefs.proxy_pass, len_p);
+
+		send (sok, buf, 3 + len_u + len_p, 0);
+		if ( recv (sok, buf, 2, 0) != 2 )
+			return 1;
+		if ( buf[1] != 0 )
+			return 1; /* UPA failed! */
+	}
+	else
+	{
+		if (buf[0] != 5 || buf[1] != 0)
+			return 1;
+	}
 
 	addrlen = strlen (serverAddr);
 	packetlen = 4 + 1 + addrlen + 2;
@@ -1095,7 +1180,7 @@ traverse_socks5 (int sok, char *serverAddr, int port)
 	{
 		if (recv (sok, buf, 1, 0) != 1)
 			return 1;
-		packetlen = buf[0] + 2;
+		packetlen = buf[0] + 2;	/* can't exceed 260 */
 		if (recv (sok, buf, packetlen, 0) != packetlen)
 			return 1;
 	}
@@ -1160,7 +1245,35 @@ base64_encode (char *to, char *from, unsigned int len)
 }
 
 static int
-traverse_http (int sok, char *serverAddr, int port)
+http_read_line (int print_fd, int sok, char *buf, int len)
+{
+#ifdef WIN32
+	/* make sure waitline() uses recv() or it'll fail on win32 */
+	len = waitline2 (source, buf, len);
+#else
+	len = waitline (sok, buf, len, TRUE);
+#endif
+	if (len >= 1)
+	{
+		/* print the message out (send it to the parent process) */
+		write (print_fd, "0\n", 2);
+
+		if (buf[len-1] == '\r')
+		{
+			buf[len-1] = '\n';
+			write (print_fd, buf, len);
+		} else
+		{
+			write (print_fd, buf, len);
+			write (print_fd, "\n", 1);
+		}
+	}
+
+	return len;
+}
+
+static int
+traverse_http (int print_fd, int sok, char *serverAddr, int port)
 {
 	char buf[256];
 	char auth_data[128];
@@ -1179,27 +1292,24 @@ traverse_http (int sok, char *serverAddr, int port)
 	n += snprintf (buf+n, sizeof (buf)-n, "\r\n");
 	send (sok, buf, n, 0);
 
-	/* dont eat this, it's actually informative to display */
-#if 0
-	waitline (sok, buf, sizeof (buf)); /* FIXME: win32 cant read() sok */
+	n = http_read_line (print_fd, sok, buf, sizeof (buf));
 	/* "HTTP/1.0 200 OK" */
-	if (strlen (buf) < 12)
+	if (n < 12)
 		return 1;
 	if (memcmp (buf, "HTTP/", 5) || memcmp (buf + 9, "200", 3))
 		return 1;
-	for (;;)
+	while (1)
 	{
 		/* read until blank line */
-		waitline (sok, buf, sizeof (buf));
-		if (!buf[0] || (buf[0] == '\r' && !buf[1]))
+		n = http_read_line (print_fd, sok, buf, sizeof (buf));
+		if (n < 1 || (n == 1 && buf[0] == '\n'))
 			break;
 	}
-#endif
 	return 0;
 }
 
 static int
-traverse_proxy (int sok, char *ip, int port)
+traverse_proxy (int print_fd, int sok, char *ip, int port)
 {
 	switch (prefs.proxy_type)
 	{
@@ -1210,7 +1320,7 @@ traverse_proxy (int sok, char *ip, int port)
 	case 3:
 		return traverse_socks5 (sok, ip, port);
 	case 4:
-		return traverse_http (sok, ip, port);
+		return traverse_http (print_fd, sok, ip, port);
 	}
 
 	return 1;
@@ -1233,9 +1343,7 @@ server_child (server * serv)
 	char *proxy_ip = NULL;
 	char *local_ip;
 	int connect_port;
-	FILE *fd;
-
-	fd = fdopen (serv->childwrite, "w");
+	char buf[512];
 
 	ns_server = net_store_new ();
 
@@ -1246,26 +1354,26 @@ server_child (server * serv)
 		local_ip = net_resolve (ns_local, prefs.hostname, 0, &real_hostname);
 		if (local_ip != NULL)
 		{
-			fprintf (fd, "5\n%s\n", local_ip);
+			snprintf (buf, sizeof (buf), "5\n%s\n", local_ip);
+			write (serv->childwrite, buf, strlen (buf));
 			net_bind (ns_local, serv->sok4, serv->sok6);
 		} else
 		{
-			fprintf (fd, "7\n");
+			write (serv->childwrite, "7\n", 2);
 		}
 		net_store_destroy (ns_local);
-		fflush (fd);
 	}
 
 	/* first resolve where we want to connect to */
 	if (!serv->dont_use_proxy && prefs.proxy_host[0] && prefs.proxy_type > 0)
 	{
-		fprintf (fd, "9\n%s\n", prefs.proxy_host);
-		fflush (fd);
+		snprintf (buf, sizeof (buf), "9\n%s\n", prefs.proxy_host);
+		write (serv->childwrite, buf, strlen (buf));
 		ip = net_resolve (ns_server, prefs.proxy_host, prefs.proxy_port,
 								&real_hostname);
 		if (!ip)
 		{
-			fprintf (fd, "1\n");
+			write (serv->childwrite, "1\n", 2);
 			goto xit;
 		}
 		connect_port = prefs.proxy_port;
@@ -1277,7 +1385,7 @@ server_child (server * serv)
 			proxy_ip = net_resolve (ns_proxy, hostname, port, &real_hostname);
 			if (!proxy_ip)
 			{
-				fprintf (fd, "1\n");
+				write (serv->childwrite, "1\n", 2);
 				goto xit;
 			}
 		} else						  /* otherwise we can just use the hostname */
@@ -1287,45 +1395,46 @@ server_child (server * serv)
 		ip = net_resolve (ns_server, hostname, port, &real_hostname);
 		if (!ip)
 		{
-			fprintf (fd, "1\n");
+			write (serv->childwrite, "1\n", 2);
 			goto xit;
 		}
 		connect_port = port;
 	}
 
-	fprintf (fd, "3\n%s\n%s\n%d\n", real_hostname, ip, connect_port);
-	fflush (fd);
+	snprintf (buf, sizeof (buf), "3\n%s\n%s\n%d\n",
+				 real_hostname, ip, connect_port);
+	write (serv->childwrite, buf, strlen (buf));
 
 	error = net_connect (ns_server, serv->sok4, serv->sok6, &sok);
 
 	if (error != 0)
 	{
-		fprintf (fd, "2\n%d\n", sock_error ());
+		snprintf (buf, sizeof (buf), "2\n%d\n", sock_error ());
+		write (serv->childwrite, buf, strlen (buf));
 	} else
 	{
 		/* connect succeeded */
 		if (proxy_ip)
 		{
-			switch (traverse_proxy (sok, proxy_ip, port))
+			switch (traverse_proxy (serv->childwrite, sok, proxy_ip, port))
 			{
-			case 0:
-				fprintf (fd, "4\n%d\n", sok);	/* success */
+			case 0:	/* success */
+				snprintf (buf, sizeof (buf), "4\n%d\n", sok);	/* success */
+				write (serv->childwrite, buf, strlen (buf));
 				break;
-			case 1:
-				fprintf (fd, "8\n");	  /* socks traversal failed */
+			case 1:	/* socks traversal failed */
+				write (serv->childwrite, "8\n", 2);
 				break;
 			}
 		} else
 		{
-			fprintf (fd, "4\n%d\n", sok);
+			snprintf (buf, sizeof (buf), "4\n%d\n", sok);	/* success */
+			write (serv->childwrite, buf, strlen (buf));
 		}
 	}
 
 xit:
 
-	/* 'fd' is leaked on win32 */
-	fflush (fd);
-
 #if defined (USE_IPV6) || defined (WIN32)
 	/* this is probably not needed */
 	net_store_destroy (ns_server);
@@ -1342,7 +1451,6 @@ xit:
 		free (ip);
 	if (real_hostname)
 		free (real_hostname);
-	/*fclose (fd);*/
 #endif
 
 	return 0;
