@@ -45,8 +45,8 @@ static const struct defaultserver def[] =
 {
 	{"2600net",	0},
 	{0,			"irc.2600.net"},
-	{0,			"irc.dc2600.com"},
-	{0,			"irc.nyc2600.com"},
+	{0,			"ptolomea.2600.net"},
+	{0,			"collective.2600.net"},
 
 	{"AbleNET",	0},
 	{0,			"california.ablenet.org"},
@@ -55,6 +55,10 @@ static const struct defaultserver def[] =
 	{0,			"extreme.ablenet.org"},
 	{0,			"irc.ablenet.org"},
 
+	{"AccessIRC",	0},
+	{0,			"irc.accessirc.net"},
+	{0,			"eu.accessirc.net"},
+
 	{"AfterNET",	0},
 	{0,			"irc.afternet.org"},
 	{0,			"us.afternet.org"},
@@ -62,7 +66,13 @@ static const struct defaultserver def[] =
 
 	{"Aitvaras",	0},
 #ifdef USE_IPV6
+#ifdef USE_OPENSSL
+	{0,			"irc6.ktu.lt/+7668"},
+#endif
 	{0,			"irc6.ktu.lt/7666"},
+#endif
+#ifdef USE_OPENSSL
+	{0,			"irc.data.lt/+6668"},
 #endif
 	{0,			"irc.data.lt"},
 	{0,			"irc.ktu.lt"},
@@ -149,7 +159,6 @@ static const struct defaultserver def[] =
 	{0,			"irc.Prison.NET"},
 	{0,			"irc.Qeast.net"},
 	{0,			"irc.efnet.pl"},
-	{0,			"irc.flamed.net"},
 	{0,			"efnet.demon.co.uk"},
 	{0,			"irc.lagged.org"},
 	{0,			"irc.lightning.net"},
@@ -184,9 +193,6 @@ static const struct defaultserver def[] =
 
 	{"FreeNode",	0},
 	{0,				"irc.freenode.net"},
-	{0,				"irc.eu.freenode.net"},
-	{0,				"irc.au.freenode.net"},
-	{0,				"irc.us.freenode.net"},
 
 	{"Freeworld",	0},
 	{0,			"kabel.freeworld.nu"},
@@ -258,11 +264,6 @@ static const struct defaultserver def[] =
 	{0,			"irc.canadian.net"},
 	{0,			"irc.irctoo.net"},
 
-	{"KewlNet",	0},
-	{0,			"random.irc.kewl.org"},
-	{0,			"la.defense.fr.eu.kewl.org"},
-	{0,			"nanterre.fr.eu.kewl.org"},
-
 	{"Librenet",	0},
 	{0,			"irc.librenet.net"},
 	{0,			"ielf.fr.librenet.net"},
@@ -278,6 +279,9 @@ static const struct defaultserver def[] =
 	{"MagicStar",	0},
 	{0,			"irc.magicstar.net"},
 
+	{"Majistic",	0},
+	{0,			"irc.majistic.net"},
+
 	{"NeverNET",	0},
 	{0,			"irc.nevernet.net"},
 	{0,			"imagine.nevernet.net"},
@@ -408,6 +412,9 @@ static const struct defaultserver def[] =
 	{0,			"moo.slashnet.org"},
 	{0,			"radon.slashnet.org"},
 
+	{"Sohbet.Net", 0},
+	{0,			"irc.sohbet.net"},
+
 	{"SorceryNet",	0},
 	{0,			"irc.sorcery.net/9000"},
 	{0,			"irc.us.sorcery.net/9000"},
@@ -454,6 +461,10 @@ static const struct defaultserver def[] =
 	{"Unsecurity",	0},
 	{0,			"irc.unsecurity.org"},
 
+	{"Worldnet",		0},
+	{0,			"irc.worldnet.net"},
+	{0,			"irc.fr.worldnet.net"},
+
 	{"Xentonix.net",	0},
 	{0,			"irc.ffm.de.eu.xentonix.net"},
 	{0,			"irc.brs.de.eu.xentonix.net"},
@@ -485,7 +496,7 @@ servlist_connect (session *sess, ircnet *net)
 	server *serv;
 
 	if (!sess)
-		sess = new_ircwindow (NULL, NULL, SESS_SERVER);
+		sess = new_ircwindow (NULL, NULL, SESS_SERVER, 0);
 
 	serv = sess->server;
 
@@ -498,23 +509,12 @@ servlist_connect (session *sess, ircnet *net)
 	/* incase a protocol switch is added to the servlist gui */
 	server_fill_her_up (sess->server);
 
-	if (net->autojoin)
+	if (net->autojoin && !sess->server->recondelay_tag)
 		safe_strcpy (sess->willjoinchannel, net->autojoin,
 						 sizeof (sess->willjoinchannel));
 	if (net->pass)
 		safe_strcpy (serv->password, net->pass, sizeof (serv->password));
 
-	if (serv->username)
-	{
-		free (serv->username);
-		serv->username = NULL;
-	}
-	if (serv->realname)
-	{
-		free (serv->realname);
-		serv->realname = NULL;
-	}
-
 	if (net->flags & FLAG_USE_GLOBAL)
 	{
 		strcpy (serv->nick, prefs.nick1);
@@ -522,10 +522,6 @@ servlist_connect (session *sess, ircnet *net)
 	{
 		if (net->nick)
 			strcpy (serv->nick, net->nick);
-		if (net->user)
-			serv->username = strdup (net->user);
-		if (net->real)
-			serv->realname = strdup (net->real);
 	}
 
 	serv->dont_use_proxy = (net->flags & FLAG_USE_PROXY) ? FALSE : TRUE;
@@ -536,13 +532,7 @@ servlist_connect (session *sess, ircnet *net)
 		(net->flags & FLAG_ALLOW_INVALID) ? TRUE : FALSE;
 #endif
 
-	if (net->flags & FLAG_CYCLE)
-		/* needed later to do cycling */
-		serv->network = net;
-	else
-		serv->network = NULL;
-
-	serv->networkname = strdup (net->name);
+	serv->network = net;
 
 	port = strrchr (ircserv->hostname, '/');
 	if (port)
@@ -565,13 +555,6 @@ servlist_connect (session *sess, ircnet *net)
 	} else
 		serv->connect (serv, ircserv->hostname, 6667, FALSE);
 
-	if (net->command)
-	{
-		if (serv->eom_cmd)
-			free (serv->eom_cmd);
-		serv->eom_cmd = strdup (net->command[0] == '/' ? net->command+1 : net->command);
-	}
-
 	if (serv->encoding)
 	{
 		free (serv->encoding);
@@ -673,9 +656,13 @@ servlist_cycle (server *serv)
 		max = g_slist_length (net->servlist);
 		if (max > 0)
 		{
-			net->selected++;
-			if (net->selected >= max)
-				net->selected = 0;
+			/* try the next server, if that option is on */
+			if (net->flags & FLAG_CYCLE)
+			{
+				net->selected++;
+				if (net->selected >= max)
+					net->selected = 0;
+			}
 
 			del = prefs.recon_delay * 1000;
 			if (del < 1000)
@@ -794,6 +781,8 @@ servlist_net_remove (ircnet *net)
 		free (net->autojoin);
 	if (net->command)
 		free (net->command);
+	if (net->nickserv)
+		free (net->nickserv);
 	if (net->comment)
 		free (net->comment);
 	if (net->encoding)
@@ -813,7 +802,7 @@ servlist_net_remove (ircnet *net)
 }
 
 ircnet *
-servlist_net_add (char *name, char *comment)
+servlist_net_add (char *name, char *comment, int prepend)
 {
 	ircnet *net;
 
@@ -823,7 +812,10 @@ servlist_net_add (char *name, char *comment)
 /*	net->comment = strdup (comment);*/
 	net->flags = FLAG_CYCLE | FLAG_USE_GLOBAL | FLAG_USE_PROXY;
 
-	network_list = g_slist_append (network_list, net);
+	if (prepend)
+		network_list = g_slist_prepend (network_list, net);
+	else
+		network_list = g_slist_append (network_list, net);
 
 	return net;
 }
@@ -838,7 +830,12 @@ servlist_load_defaults (void)
 	{
 		if (def[i].network)
 		{
-			net = servlist_net_add (def[i].network, def[i].host);
+			net = servlist_net_add (def[i].network, def[i].host, FALSE);
+#ifdef WIN32
+			/* Windows gets UTF-8 for new users. Unix gets "System Default",
+				which is often UTF-8 anyway! */
+			net->encoding = strdup ("UTF-8");
+#endif
 			if (def[i].channel)
 				net->autojoin = strdup (def[i].channel);
 			if (!strcmp (def[i].network, "ChatJunkies"))
@@ -860,6 +857,7 @@ servlist_load (void)
 	FILE *fp;
 	char buf[258];
 	int len;
+	char *tmp;
 	ircnet *net = NULL;
 
 	snprintf (buf, sizeof (buf), "%s/servlist_.conf", get_xdir_fs ());
@@ -892,7 +890,17 @@ servlist_load (void)
 				net->autojoin = strdup (buf + 2);
 				break;
 			case 'C':
-				net->command = strdup (buf + 2);
+				if (net->command)
+				{
+					/* concat extra commands with a \n separator */
+					tmp = net->command;
+					net->command = malloc (strlen (tmp) + strlen (buf + 2) + 2);
+					strcpy (net->command, tmp);
+					strcat (net->command, "\n");
+					strcat (net->command, buf + 2);
+					free (tmp);
+				} else
+					net->command = strdup (buf + 2);
 				break;
 			case 'F':
 				net->flags = atoi (buf + 2);
@@ -906,10 +914,13 @@ servlist_load (void)
 			case 'S':	/* new server/hostname for this network */
 				servlist_server_add (net, buf + 2);
 				break;
+			case 'B':
+				net->nickserv = strdup (buf + 2);
+				break;
 			}
 		}
 		if (buf[0] == 'N')
-			net = servlist_net_add (buf + 2, /* comment */ "");
+			net = servlist_net_add (buf + 2, /* comment */ NULL, FALSE);
 	}
 	fclose (fp);
 
@@ -949,7 +960,49 @@ servlist_check_encoding (char *charset)
 	return FALSE;
 }
 
-void
+static int
+servlist_write_ccmd (char *str, void *fp)
+{
+	return fprintf (fp, "C=%s\n", (str[0] == '/') ? str + 1 : str);
+}
+
+/* separates a string according to a 'sep' char, then calls the callback
+   function for each token found */
+
+int
+token_foreach (char *str, char sep,
+					int (*callback) (char *str, void *ud), void *ud)
+{
+	char t, *start = str;
+
+	while (1)
+	{
+		if (*str == sep || *str == 0)
+		{
+			t = *str;
+			*str = 0;
+			if (callback (start, ud) < 1)
+			{
+				*str = t;
+				return FALSE;
+			}
+			*str = t;
+
+			if (*str == 0)
+				break;
+			str++;
+			start = str;
+
+		} else
+		{
+			str += g_utf8_skip[*((unsigned char *)str)];
+		}
+	}
+
+	return TRUE;
+}
+
+int
 servlist_save (void)
 {
 	FILE *fp;
@@ -965,7 +1018,7 @@ servlist_save (void)
 		first = TRUE;
 	fp = fopen (buf, "w");
 	if (!fp)
-		return;
+		return FALSE;
 
 #ifndef WIN32
 	if (first)
@@ -989,6 +1042,8 @@ servlist_save (void)
 			fprintf (fp, "P=%s\n", net->pass);
 		if (net->autojoin)
 			fprintf (fp, "J=%s\n", net->autojoin);
+		if (net->nickserv)
+			fprintf (fp, "B=%s\n", net->nickserv);
 		if (net->encoding && strcasecmp (net->encoding, "System") &&
 			 strcasecmp (net->encoding, "System default"))
 		{
@@ -1000,8 +1055,10 @@ servlist_save (void)
 				fe_message (buf, FALSE);
 			}
 		}
+
 		if (net->command)
-			fprintf (fp, "C=%s\n", net->command[0] == '/' ? net->command+1 : net->command);
+			token_foreach (net->command, '\n', servlist_write_ccmd, fp);
+
 		fprintf (fp, "F=%d\nD=%d\n", net->flags, net->selected);
 
 		hlist = net->servlist;
@@ -1011,10 +1068,16 @@ servlist_save (void)
 			fprintf (fp, "S=%s\n", serv->hostname);
 			hlist = hlist->next;
 		}
-		fprintf (fp, "\n");
+
+		if (fprintf (fp, "\n") < 1)
+		{
+			fclose (fp);
+			return FALSE;
+		}
 
 		list = list->next;
 	}
 
 	fclose (fp);
+	return TRUE;
 }
