@@ -47,103 +47,6 @@ struct pevt_stage1
 };
 
 
-#ifdef USE_HEBREW
-
-static inline int iswhitespace(const char ch)
-{
-	return (ch == ' ' ||  ch == 8);
-}
-
-static inline int isheb(const char ch)
-{
-	return (((unsigned char)ch) >= 0xE0  &&  ((unsigned char)ch) <= 0xFA);
-}
-
-static inline int iseng(const char ch)
-{
-	return ((ch >= 'A'  &&  ch <= 'z')  ||  (ch >= 'a'  &&  ch <= 'z'));
-}
-
-static char *strhebpatch(char *dest, const char *src)
-{
-	short int mode = 0, imode;
-	const char *hmark = NULL, *lmark, *nmark, *nlmark;
-	char ch;
-
-	if (!dest || !src)
-		return NULL;
-	
-	for (;;)
-	{
-		if (mode == 0)
-		{
-			if (isheb(*src))
-			{
-				hmark = src;
-				mode = 1;
-			}
-			else
-				*dest++ = *src;
-		}
-		else if (mode == 1)
-		{
-			if (*src == 0  ||  iseng(*src))
-			{
-				lmark = src-1;
-				while (!isheb(*lmark)) lmark--;
-				src = lmark;
-				imode = 0;
-				nmark = NULL;
-
-				while (lmark >= hmark)
-				{
-					ch = *lmark;
-
-					if (imode == 0)
-						switch (ch)
-						{
-							case '(': ch = ')'; break;
-							case ')': ch = '('; break;
-							case '{': ch = '}'; break;
-							case '}': ch = '{'; break;
-							case '[': ch = ']'; break;
-							case ']': ch = '['; break;
-						}
-
-					if (imode == 0)
-					{
-						if (isdigit(ch))
-						{
-							imode = 1;
-							nmark = lmark;			
-						}
-						else 
-							*dest++ = ch;
-					}
-					else
-  						if (imode == 1  &&  (isheb(ch) || iswhitespace(ch)))
-						{
-							nlmark = lmark+1;
-							while (nlmark <= nmark)
-								*dest++ = *nlmark++;
-							imode = 0;
-							lmark++;
-						}
-					lmark--;
-				}
-				
-				hmark = NULL;
-				mode = 0;
-			}
-		}
-		if (!*src++)
-			break;
-	}
-	return dest;
-}
-
-#endif /* !USE_HEBREW */
-
 /* Make ip:port urls clikable (127.0.0.1:80 etc)
 *  patch by Alex <alex@cosinus.org> & dobler <dobler@barrysworld.com>
 */
@@ -199,7 +102,7 @@ text_word_check (char *word)
 	if(q3link(word))
 		return WORD_URL;
 
-	if ((word[0] == '@' || word[0] == '+') && word[1] == '#')
+	if ((word[0] == '@' || word[0] == '+' || word[0] == '^' || word[0] == '%' || word[0] == '*' ) && word[1] == '#')
 		return WORD_CHANNEL;
 
 	if ((word[0] == '#' || word[0] == '&') && word[1] != '#' && word[1] != 0)
@@ -330,8 +233,9 @@ log_create_filename (char *buf, char *servname, char *channame, char *netname)
 				*tmp = rfc_tolower (*tmp);
 				if (*tmp == '/')
 #else
-				/* win32 can't handle filenames with \|/ characters */
-				if (*tmp == '\\' || *tmp == '|' || *tmp == '/')
+				/* win32 can't handle filenames with \|/>< characters */
+				if (*tmp == '\\' || *tmp == '|' || *tmp == '/' ||
+					 *tmp == '>' || *tmp == '<')
 #endif
 					*tmp = '_';
 			}
@@ -431,7 +335,7 @@ log_open (session *sess)
 
 	log_close (sess);
 	sess->logfd = log_open_file (sess->server->servername, sess->channel,
-										  sess->server->networkname);
+										  get_network (sess, FALSE));
 
 	if (!log_error && sess->logfd == -1)
 	{
@@ -513,9 +417,18 @@ text_validate (char **text, int *len)
 	if (g_utf8_validate (*text, *len, 0))
 		return NULL;
 
-	/* maybe it's iso-8859-1 */
-	utf = g_convert (*text, *len, "UTF-8", "ISO-8859-1", 0, &utf_len, &error);
-	if (!utf)       /* should never happen; all text is iso-8859-1 valid */
+	if (prefs.utf8_locale)
+		/* fallback to iso-8859-1 */
+		utf = g_convert (*text, *len, "UTF-8", "ISO-8859-1", 0, &utf_len, &error);
+	else
+	{
+		/* fallback to locale */
+		utf = g_locale_to_utf8 (*text, *len, 0, &utf_len, NULL);
+		if (!utf)
+			utf = g_convert (*text, *len, "UTF-8", "ISO-8859-1", 0, &utf_len, &error);
+	}
+
+	if (!utf) 
 	{
 		if (error)
 		{
@@ -560,27 +473,7 @@ PrintText (session *sess, char *text)
 	}
 
 	log_write (sess, text);
-
-#ifdef USE_HEBREW
-	if (prefs.hebrew)
-	{
-   	const char *orgtext = NULL;
-   	if (text)
-   	{
-			orgtext = text;
-			text = strdup(text);
-			strhebpatch (text, orgtext);
-   	}
-		fe_print_text (sess, text);
-   	if (orgtext)
-			free(text);
-	} else
-	{
-#endif
-		fe_print_text (sess, text);
-#ifdef USE_HEBREW
-	}
-#endif
+	fe_print_text (sess, text);
 
 	if (conv)
 		g_free (conv);
@@ -682,6 +575,7 @@ static char *pevt_chanmsg_help[] = {
 	N_("Nickname"),
 	N_("The text"),
 	N_("Mode char"),
+	N_("Identified text"),
 };
 
 static char *pevt_privmsg_help[] = {
@@ -754,6 +648,7 @@ static char *pevt_channotice_help[] = {
 static char *pevt_dprivmsg_help[] = {
 	N_("Nickname"),
 	N_("The message"),
+	N_("Identified text"),
 };
 
 static char *pevt_uchangenick_help[] = {
@@ -1148,16 +1043,18 @@ static char *pevt_discon_help[] = {
 static void
 pevent_load_defaults ()
 {
-	int i, len;
+	int i;
 
 	for (i = 0; i < NUM_XP; i++)
 	{
-		len = strlen (_(te[i].def));
-		len++;
 		if (pntevts_text[i])
 			free (pntevts_text[i]);
-		pntevts_text[i] = malloc (len);
-		memcpy (pntevts_text[i], _(te[i].def), len);
+
+		/* don't gettext() the blank ones */
+		if (i == XP_TE_OPENDIALOG || i == XP_TE_BEEP)
+			pntevts_text[i] = strdup (te[i].def);
+		else
+			pntevts_text[i] = strdup (_(te[i].def));
 	}
 }
 
@@ -1174,7 +1071,7 @@ pevent_make_pntevts ()
 		if (pevt_build_string (pntevts_text[i], &(pntevts[i]), &m) != 0)
 		{
 			snprintf (out, sizeof (out),
-						 _("Error parsing event %s.\nLoading default"), te[i].name);
+						 _("Error parsing event %s.\nLoading default."), te[i].name);
 			fe_message (out, FALSE);
 			free (pntevts_text[i]);
 			len = strlen (te[i].def) + 1;
@@ -1187,7 +1084,6 @@ pevent_make_pntevts ()
 				abort ();
 			}
 		}
-		check_special_chars (pntevts[i], TRUE);
 	}
 }
 
@@ -1212,15 +1108,6 @@ pevent_trigger_load (int *i_penum, char **i_text, char **i_snd)
 			free (pntevts_text[penum]);
 		pntevts_text[penum] = malloc (len);
 		memcpy (pntevts_text[penum], text, len);
-		if (te[penum].sound)
-			free (te[penum].sound);
-		if (snd)
-		{
-			len = strlen (snd) + 1;
-			te[penum].sound = malloc (len);
-			memcpy (te[penum].sound, snd, len);
-		} else
-			te[penum].sound = NULL;
 	}
 
 	if (text)
@@ -1309,6 +1196,7 @@ pevent_load (char *filename)
 			if (text)
 				free (text);
 
+#if 0
 			/* This allows updating of old strings. We don't use new defaults
 				if the user has customized the strings (.e.g a text theme).
 				Hash of the old default is enough to identify and replace it.
@@ -1334,15 +1222,18 @@ pevent_load (char *filename)
 			default:
 				text = strdup (ofs);
 			}
+#else
+			text = strdup (ofs);
+#endif
 
 			continue;
-		} else if (strcmp (buf, "event_sound") == 0)
+		}/* else if (strcmp (buf, "event_sound") == 0)
 		{
 			if (snd)
 				free (snd);
 			snd = strdup (ofs);
 			continue;
-		}
+		}*/
 
 		continue;
 	}
@@ -1635,62 +1526,6 @@ pevt_build_string (const char *input, char **output, int *max_arg)
 	return 0;
 }
 
-static void
-play_wave (const char *file)
-{
-	char buf[512];
-	char wavfile[512];
-	char *file_fs;
-
-	/* the pevents GUI editor triggers this after removing a soundfile */
-	if (!file[0])
-		return;
-
-	memset (buf, 0, sizeof (buf));
-	memset (wavfile, 0, sizeof (wavfile));
-
-#ifdef WIN32
-	/* check for fullpath, windows style */
-	if (strlen (file) > 3 &&
-		 file[1] == ':' && (file[2] == '\\' || file[2] == '/') )
-	{
-		strncpy (wavfile, file, sizeof (wavfile));
-	} else
-#endif
-	if (file[0] != '/')
-	{
-		snprintf (wavfile, sizeof (wavfile), "%s/%s", prefs.sounddir, file);
-	} else
-	{
-		strncpy (wavfile, file, sizeof (wavfile));
-	}
-
-	file_fs = g_filename_from_utf8 (wavfile, -1, 0, 0, 0);
-	if (!file_fs)
-		return;
-
-	if (access (file_fs, R_OK) == 0)
-	{
-#ifdef WIN32
-		if (strcmp (prefs.soundcmd, "esdplay") == 0)
-		{
-			PlaySound (file_fs, NULL, SND_NODEFAULT|SND_FILENAME|SND_ASYNC);
-		} else
-#endif
-		{
-			snprintf (buf, sizeof (buf), "%s %s", prefs.soundcmd, file_fs);
-			buf[sizeof (buf) - 1] = '\0';
-			xchat_exec (buf);
-		}
-	} else
-	{
-		snprintf (buf, sizeof (buf), _("Cannot read sound file:\n%s"), wavfile);
-		fe_message (buf, FALSE);
-	}
-
-	g_free (file_fs);
-}
-
 /* called by EMIT_SIGNAL macro */
 
 void
@@ -1719,8 +1554,7 @@ text_emit (int index, session *sess, char *a, char *b, char *c, char *d)
 	if (plugin_emit_print (sess, word))
 		return;
 
-	if (te[index].sound)
-		play_wave (te[index].sound);
+	sound_play_event (index);
 
 	display_event (pntevts[index], sess, te[index].num_args, word);
 }
@@ -1768,11 +1602,194 @@ pevent_save (char *fn)
 		write (fd, buf, snprintf (buf, sizeof (buf),
 										  "event_name=%s\n", te[i].name));
 		write (fd, buf, snprintf (buf, sizeof (buf),
-										  "event_text=%s\n", pntevts_text[i]));
-		if (te[i].sound && te[i].sound[0])
+										  "event_text=%s\n\n", pntevts_text[i]));
+	}
+
+	close (fd);
+}
+
+/* =========================== */
+/* ========== SOUND ========== */
+/* =========================== */
+
+char *sound_files[NUM_XP];
+
+void
+sound_beep (session *sess)
+{
+	if (sound_files[XP_TE_BEEP] && sound_files[XP_TE_BEEP][0])
+		/* user defined beep _file_ */
+		sound_play_event (XP_TE_BEEP);
+	else
+		/* system beep */
+		fe_beep ();
+}
+
+static char *
+sound_find_command (void)
+{
+	/* some sensible unix players. You're bound to have one of them */
+	static const char *progs[] = {"aplay", "esdplay", "soxplay", "artsplay", NULL};
+	char *cmd;
+	int i = 0;
+
+	if (prefs.soundcmd[0])
+		return g_strdup (prefs.soundcmd);
+
+	while (progs[i])
+	{
+		cmd = g_find_program_in_path (progs[i]);
+		if (cmd)
+			return cmd;
+		i++;
+	}
+
+	return NULL;
+}
+
+void
+sound_play (const char *file)
+{
+	char buf[512];
+	char wavfile[512];
+	char *file_fs;
+	char *cmd;
+
+	/* the pevents GUI editor triggers this after removing a soundfile */
+	if (!file[0])
+		return;
+
+	memset (buf, 0, sizeof (buf));
+	memset (wavfile, 0, sizeof (wavfile));
+
+#ifdef WIN32
+	/* check for fullpath, windows style */
+	if (strlen (file) > 3 &&
+		 file[1] == ':' && (file[2] == '\\' || file[2] == '/') )
+	{
+		strncpy (wavfile, file, sizeof (wavfile));
+	} else
+#endif
+	if (file[0] != '/')
+	{
+		snprintf (wavfile, sizeof (wavfile), "%s/%s", prefs.sounddir, file);
+	} else
+	{
+		strncpy (wavfile, file, sizeof (wavfile));
+	}
+
+	file_fs = g_filename_from_utf8 (wavfile, -1, 0, 0, 0);
+	if (!file_fs)
+		return;
+
+	if (access (file_fs, R_OK) == 0)
+	{
+		cmd = sound_find_command ();
+
+#ifdef WIN32
+		if (cmd == NULL || strcmp (cmd, "esdplay") == 0)
+		{
+			PlaySound (file_fs, NULL, SND_NODEFAULT|SND_FILENAME|SND_ASYNC);
+		} else
+#endif
+		{
+			if (cmd)
+			{
+				if (strchr (file_fs, ' '))
+					snprintf (buf, sizeof (buf), "%s \"%s\"", cmd, file_fs);
+				else
+					snprintf (buf, sizeof (buf), "%s %s", cmd, file_fs);
+				buf[sizeof (buf) - 1] = '\0';
+				xchat_exec (buf);
+			}
+		}
+
+		if (cmd)
+			g_free (cmd);
+
+	} else
+	{
+		snprintf (buf, sizeof (buf), _("Cannot read sound file:\n%s"), wavfile);
+		fe_message (buf, FALSE);
+	}
+
+	g_free (file_fs);
+}
+
+void
+sound_play_event (int i)
+{
+	if (sound_files[i])
+		sound_play (sound_files[i]);
+}
+
+static void
+sound_load_event (char *evt, char *file)
+{
+	int i = 0;
+
+	if (file[0] && pevent_find (evt, &i) != -1)
+	{
+		if (sound_files[i])
+			free (sound_files[i]);
+		sound_files[i] = strdup (file);
+	}
+}
+
+void
+sound_load ()
+{
+	int fd;
+	char buf[512];
+	char evt[128];
+
+	memset (&sound_files, 0, sizeof (char *) * (NUM_XP));
+
+	snprintf (buf, sizeof (buf), "%s/sound.conf", get_xdir_fs ());
+	fd = open (buf, O_RDONLY | OFLAGS);
+	if (fd == -1)
+		return;
+
+	evt[0] = 0;
+	while (waitline (fd, buf, sizeof buf, FALSE) != -1)
+	{
+		if (strncmp (buf, "event=", 6) == 0)
+		{
+			safe_strcpy (evt, buf + 6, sizeof (evt));
+		}
+		else if (strncmp (buf, "sound=", 6) == 0)
+		{
+			if (evt[0] != 0)
+			{
+				sound_load_event (evt, buf + 6);
+				evt[0] = 0;
+			}
+		}
+	}
+
+	close (fd);
+}
+
+void
+sound_save ()
+{
+	int fd, i;
+	char buf[512];
+
+	snprintf (buf, sizeof (buf), "%s/sound.conf", get_xdir_fs ());
+	fd = open (buf, O_CREAT | O_TRUNC | O_WRONLY | OFLAGS, 0x180);
+	if (fd == -1)
+		return;
+
+	for (i = 0; i < NUM_XP; i++)
+	{
+		if (sound_files[i] && sound_files[i][0])
+		{
 			write (fd, buf, snprintf (buf, sizeof (buf),
-											  "event_sound=%s\n", te[i].sound));
-		write (fd, "\n", 1);
+											  "event=%s\n", te[i].name));
+			write (fd, buf, snprintf (buf, sizeof (buf),
+											  "sound=%s\n\n", sound_files[i]));
+		}
 	}
 
 	close (fd);
