@@ -194,7 +194,7 @@ away_check (void)
 	GSList *list;
 	int full, sent, loop = 0;
 
-	if (prefs.away_size_max < 1)
+	if (!prefs.away_track || prefs.away_size_max < 1)
 		return 1;
 
 doover:
@@ -305,7 +305,7 @@ irc_init (session *sess)
 }
 
 static session *
-new_session (server *serv, char *from, int type)
+new_session (server *serv, char *from, int type, int focus)
 {
 	session *sess;
 
@@ -322,7 +322,7 @@ new_session (server *serv, char *from, int type)
 
 	sess_list = g_slist_prepend (sess_list, sess);
 
-	fe_new_window (sess);
+	fe_new_window (sess, focus);
 
 	return sess;
 }
@@ -350,11 +350,13 @@ set_server_defaults (server *serv)
 	serv->nick_modes = strdup ("ohv");
 
 	serv->nickcount = 1;
+	serv->nickservtype = 0;
 	serv->end_of_motd = FALSE;
 	serv->is_away = FALSE;
 	serv->supports_watch = FALSE;
 	serv->bad_prefix = FALSE;
 	serv->use_who = TRUE;
+	serv->have_idmsg = FALSE;
 }
 
 static server *
@@ -382,7 +384,7 @@ new_server (void)
 }
 
 session *
-new_ircwindow (server *serv, char *name, int type)
+new_ircwindow (server *serv, char *name, int type, int focus)
 {
 	session *sess;
 
@@ -394,17 +396,17 @@ new_ircwindow (server *serv, char *name, int type)
 		{
 			register unsigned int oldh = prefs.hideuserlist;
 			prefs.hideuserlist = 1;
-			sess = new_session (serv, name, SESS_SERVER);
+			sess = new_session (serv, name, SESS_SERVER, focus);
 			prefs.hideuserlist = oldh;
 		} else
 		{
-			sess = new_session (serv, name, SESS_CHANNEL);
+			sess = new_session (serv, name, SESS_CHANNEL, focus);
 		}
 		serv->server_session = sess;
 		serv->front_session = sess;
 		break;
 	case SESS_DIALOG:
-		sess = new_session (serv, name, type);
+		sess = new_session (serv, name, type, focus);
 		if (prefs.logging)
 			log_open (sess);
 		break;
@@ -412,7 +414,7 @@ new_ircwindow (server *serv, char *name, int type)
 /*	case SESS_CHANNEL:
 	case SESS_NOTICES:
 	case SESS_SNOTICES:*/
-		sess = new_session (serv, name, type);
+		sess = new_session (serv, name, type, focus);
 		break;
 	}
 
@@ -422,6 +424,18 @@ new_ircwindow (server *serv, char *name, int type)
 	return sess;
 }
 
+char *
+get_network (session *sess, gboolean fallback)
+{
+	if (sess->server->network)
+		return ((ircnet *)sess->server->network)->name;
+
+	if (fallback)
+		return sess->server->servername;
+
+	return NULL;
+}
+
 static void
 free_away_messages (server *serv)
 {
@@ -488,14 +502,6 @@ kill_server_callback (server * serv)
 		free (serv->bad_nick_prefixes);
 	if (serv->last_away_reason)
 		free (serv->last_away_reason);
-	if (serv->eom_cmd)
-		free (serv->eom_cmd);
-	if (serv->username)
-		free (serv->username);
-	if (serv->realname)
-		free (serv->realname);
-	if (serv->networkname)
-		free (serv->networkname);
 	if (serv->encoding)
 		free (serv->encoding);
 
@@ -675,16 +681,16 @@ find_away_message (struct server *serv, char *nick)
 
 #define XTERM "gnome-terminal -x "
 
-#define defaultconf_ctcp \
+static char defaultconf_ctcp[] =
 	"NAME TIME\n"				"CMD nctcp %s TIME %t\n\n"\
-	"NAME PING\n"				"CMD nctcp %s PING %d\n\n"
+	"NAME PING\n"				"CMD nctcp %s PING %d\n\n";
 
-#define defaultconf_replace \
-	"NAME teh\n"				"CMD the\n\n"
+static char defaultconf_replace[] =
+	"NAME teh\n"				"CMD the\n\n";
 /*	"NAME r\n"					"CMD are\n\n"\
 	"NAME u\n"					"CMD you\n\n"*/
 
-#define defaultconf_commands \
+static char defaultconf_commands[] =
 	"NAME ACTION\n"		"CMD me &2\n\n"\
 	"NAME AME\n"			"CMD allchan me &2\n\n"\
 	"NAME ANICK\n"			"CMD allserv nick &2\n\n"\
@@ -711,13 +717,13 @@ find_away_message (struct server *serv, char *nick)
 	"NAME VER\n"			"CMD ctcp %2 VERSION\n\n"\
 	"NAME VERSION\n"		"CMD ctcp %2 VERSION\n\n"\
 	"NAME WALLOPS\n"		"CMD quote WALLOPS :&2\n\n"\
-	"NAME WII\n"			"CMD quote WHOIS %2 %2\n\n"
+	"NAME WII\n"			"CMD quote WHOIS %2 %2\n\n";
 
 #ifdef WIN32
-#define defaultconf_urlhandlers \
-	"NAME Connect as IRC server\n"		"CMD newserver %s\n\n"
+static char defaultconf_urlhandlers[] =
+	"NAME Connect as IRC server\n"		"CMD newserver %s\n\n";
 #else
-#define defaultconf_urlhandlers \
+static char defaultconf_urlhandlers[] =
 	"NAME SUB\n"								"CMD Epiphany...\n\n"\
 		"NAME Open\n"							"CMD !epiphany '%s'\n\n"\
 		"NAME Open in new tab\n"			"CMD !epiphany -n '%s'\n\n"\
@@ -735,9 +741,9 @@ find_away_message (struct server *serv, char *nick)
 		"NAME Run new Mozilla\n"			"CMD !mozilla %s\n\n"\
 	"NAME ENDSUB\n"							"CMD \n\n"\
 	"NAME SUB\n"								"CMD Mozilla FireFox...\n\n"\
-		"NAME Open in existing\n"			"CMD !firefox -remote 'openURL(%s)'\n\n"\
-		"NAME Open in new window\n"		"CMD !firefox -remote 'openURL(%s,new-window)'\n\n"\
-		"NAME Open in new tab\n"			"CMD !firefox -remote 'openURL(%s,new-tab)'\n\n"\
+		"NAME Open in existing\n"			"CMD !firefox -a firefox -remote 'openURL(%s)'\n\n"\
+		"NAME Open in new window\n"		"CMD !firefox -a firefox -remote 'openURL(%s,new-window)'\n\n"\
+		"NAME Open in new tab\n"			"CMD !firefox -a firefox -remote 'openURL(%s,new-tab)'\n\n"\
 		"NAME Run new Mozilla FireFox\n"	"CMD !firefox %s\n\n"\
 	"NAME ENDSUB\n"							"CMD \n\n"\
 	"NAME SUB\n"								"CMD Galeon...\n\n"\
@@ -763,7 +769,7 @@ find_away_message (struct server *serv, char *nick)
 		"NAME Telnet\n"						"CMD !"XTERM"telnet %s\n\n"\
 		"NAME Ping\n"							"CMD !"XTERM"ping -c 4 %s\n\n"\
 	"NAME ENDSUB\n"							"CMD \n\n"\
-	"NAME Connect as IRC server\n"		"CMD newserver %s\n\n"
+	"NAME Connect as IRC server\n"		"CMD newserver %s\n\n";
 #endif
 
 #ifdef USE_SIGACTION
@@ -808,7 +814,7 @@ xchat_auto_connect (gpointer userdata)
 static void
 xchat_init (void)
 {
-	char buf[2048];
+	char buf[3068];
 	const char *cs = NULL;
 
 #ifdef WIN32
@@ -855,6 +861,7 @@ xchat_init (void)
 		prefs.utf8_locale = TRUE;
 
 	load_text_events ();
+	sound_load ();
 	notify_load ();
 	ignore_load ();
 
@@ -878,11 +885,14 @@ xchat_init (void)
 		"NAME %s\n"				"CMD quote KILL %%s :die!\n\n"\
 	"NAME ENDSUB\n"			"CMD \n\n"\
 	"NAME SUB\n"				"CMD %s\n\n"\
-		"NAME %s\n"				"CMD voice %%a\n\n"\
-		"NAME %s\n"				"CMD devoice %%a\n"\
-		"NAME SEP\n"			"CMD \n\n"\
 		"NAME %s\n"				"CMD op %%a\n\n"\
 		"NAME %s\n"				"CMD deop %%a\n\n"\
+		"NAME SEP\n"			"CMD \n\n"\
+		"NAME %s\n"				"CMD hop %%a\n\n"\
+		"NAME %s\n"				"CMD dehop %%a\n\n"\
+		"NAME SEP\n"			"CMD \n\n"\
+		"NAME %s\n"				"CMD voice %%a\n\n"\
+		"NAME %s\n"				"CMD devoice %%a\n"\
 	"NAME ENDSUB\n"			"CMD \n\n"\
 	"NAME SUB\n"				"CMD %s\n\n"\
 		"NAME %s\n"				"CMD ignore %%s!*@* ALL\n\n"\
@@ -928,10 +938,12 @@ xchat_init (void)
 		_("Oper"),
 		_("Kill this user"),
 		_("Mode"),
-		_("Give Voice"),
-		_("Take Voice"),
 		_("Give Ops"),
 		_("Take Ops"),
+		_("Give Half-Ops"),
+		_("Take Half-Ops"),
+		_("Give Voice"),
+		_("Take Voice"),
 		_("Ignore"),
 		_("Ignore User"),
 		_("UnIgnore User"),
@@ -999,13 +1011,13 @@ xchat_init (void)
 		"NAME %s\n"				"CMD whois %%s %%s\n\n"
 		"NAME %s\n"				"CMD dcc send %%s\n\n"
 		"NAME %s\n"				"CMD dcc chat %%s\n\n"
-		"NAME %s\n"				"CMD ping %%s\n\n"
-		"NAME %s\n"				"CMD clear\n\n",
+		"NAME %s\n"				"CMD clear\n\n"
+		"NAME %s\n"				"CMD ping %%s\n\n",
 				_("WhoIs"),
 				_("Send"),
 				_("Chat"),
-				_("Ping"),
-				_("Clear"));
+				_("Clear"),
+				_("Ping"));
 	list_loadconf ("dlgbuttons.conf", &dlgbutton_list, buf);
 
 	list_loadconf ("tabmenu.conf", &tabmenu_list, NULL);
@@ -1029,7 +1041,7 @@ xchat_init (void)
 			/* and no serverlist gui ... */
 			if (prefs.slist_skip || connect_url)
 				/* we'll have to open one. */
-				new_ircwindow (NULL, NULL, SESS_SERVER);
+				new_ircwindow (NULL, NULL, SESS_SERVER, 0);
 		} else
 		{
 			fe_idle_add (xchat_auto_connect, NULL);
@@ -1037,7 +1049,7 @@ xchat_init (void)
 	} else
 	{
 		if (prefs.slist_skip)
-			new_ircwindow (NULL, NULL, SESS_SERVER);
+			new_ircwindow (NULL, NULL, SESS_SERVER, 0);
 	}
 }
 
@@ -1051,10 +1063,12 @@ xchat_exit (void)
 	if (prefs.autosave)
 	{
 		save_config ();
-		pevent_save (NULL);
+		if (prefs.save_pevents)
+			pevent_save (NULL);
 	}
 	if (prefs.autosave_url)
 		url_autosave ();
+	sound_save ();
 	notify_save ();
 	ignore_save ();
 	free_sessions ();
