@@ -31,6 +31,7 @@
 #include <gtk/gtkbox.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtktogglebutton.h>
+#include <gtk/gtkversion.h>
 
 #include "../common/xchat.h"
 #include "../common/fe.h"
@@ -239,14 +240,14 @@ create_input_style (void)
 	if (prefs.style_inputbox && !done_rc)
 	{
 		done_rc = TRUE;
-		sprintf (buf, cursor_color_rc, (colors[18].red >> 8),
-			(colors[18].green >> 8), (colors[18].blue >> 8));
+		sprintf (buf, cursor_color_rc, (colors[COL_FG].red >> 8),
+			(colors[COL_FG].green >> 8), (colors[COL_FG].blue >> 8));
 		gtk_rc_parse_string (buf);
 	}
 
-	style->bg[GTK_STATE_NORMAL] = colors[18];
-	style->base[GTK_STATE_NORMAL] = colors[19];
-	style->text[GTK_STATE_NORMAL] = colors[18];
+	style->bg[GTK_STATE_NORMAL] = colors[COL_FG];
+	style->base[GTK_STATE_NORMAL] = colors[COL_BG];
+	style->text[GTK_STATE_NORMAL] = colors[COL_FG];
 
 	return style;
 }
@@ -276,7 +277,8 @@ fe_main (void)
 void
 fe_cleanup (void)
 {
-	palette_save ();
+	/* it's saved when pressing OK in setup.c */
+	/*palette_save ();*/
 }
 
 void
@@ -312,7 +314,7 @@ log_handler (const gchar   *log_domain,
 
 	sess = find_dialog (serv_list->data, "(warnings)");
 	if (!sess)
-		sess = new_ircwindow (serv_list->data, "(warnings)", SESS_DIALOG);
+		sess = new_ircwindow (serv_list->data, "(warnings)", SESS_DIALOG, 0);
 
 	PrintTextf (sess, "%s\t%s\n", log_domain, message);
 	if (getenv ("XCHAT_WARNING_ABORT"))
@@ -322,7 +324,7 @@ log_handler (const gchar   *log_domain,
 #endif
 
 void
-fe_new_window (session *sess)
+fe_new_window (session *sess, int focus)
 {
 	int tab = FALSE;
 
@@ -336,10 +338,10 @@ fe_new_window (session *sess)
 			tab = TRUE;
 	}
 
-	mg_changui_new (sess, NULL, tab);
+	mg_changui_new (sess, NULL, tab, focus);
 
 #ifdef WIN32
-	g_log_set_handler ("Glib", G_LOG_LEVEL_CRITICAL|G_LOG_LEVEL_WARNING, (GLogFunc)log_handler, 0);
+	g_log_set_handler ("GLib", G_LOG_LEVEL_CRITICAL|G_LOG_LEVEL_WARNING, (GLogFunc)log_handler, 0);
 	g_log_set_handler ("GLib-GObject", G_LOG_LEVEL_CRITICAL|G_LOG_LEVEL_WARNING, (GLogFunc)log_handler, 0);
 	g_log_set_handler ("Gdk", G_LOG_LEVEL_CRITICAL|G_LOG_LEVEL_WARNING, (GLogFunc)log_handler, 0);
 	g_log_set_handler ("Gtk", G_LOG_LEVEL_CRITICAL|G_LOG_LEVEL_WARNING, (GLogFunc)log_handler, 0);
@@ -694,7 +696,9 @@ fe_ctrl_gui (session *sess, int action, int arg)
 	case 0:
 		gtk_widget_hide (sess->gui->window); break;
 	case 1:
-		gtk_widget_show (sess->gui->window); break;
+		gtk_widget_show (sess->gui->window);
+		gtk_window_present (GTK_WINDOW (sess->gui->window));
+		break;
 	case 2:
 		mg_bring_tofront (sess->res->tab); break;
 	case 3:
@@ -710,3 +714,65 @@ void
 fe_confirm (const char *message, void (*yesproc)(void *), void (*noproc)(void *), void *ud)
 {
 }
+
+int
+fe_gui_info (session *sess, int info_type)
+{
+	switch (info_type)
+	{
+	case 0:	/* window status */
+		if (!GTK_WIDGET_VISIBLE (GTK_WINDOW (sess->gui->window)))
+			return 2;	/* hidden (iconified or systray) */
+#if GTK_CHECK_VERSION(2,4,0)
+		if (gtk_window_is_active (GTK_WINDOW (sess->gui->window)))
+#else
+#if GTK_CHECK_VERSION(2,2,0)
+		if (GTK_WINDOW (sess->gui->window)->is_active)
+#endif
+#endif
+			return 1;	/* active/focused */
+
+		return 0;		/* normal (no keyboard focus or behind a window) */
+	}
+
+	return -1;
+}
+
+char *
+fe_get_inputbox_contents (session *sess)
+{
+	/* not the current tab */
+	if (sess->res->input_text)
+		return sess->res->input_text;
+
+	/* current focused tab */
+	return GTK_ENTRY (sess->gui->input_box)->text;
+}
+
+void
+fe_set_inputbox_cursor (session *sess, int delta, int pos)
+{
+	if (!sess->gui->is_tab || sess == current_tab)
+	{
+		if (delta)
+			pos += gtk_editable_get_position (GTK_EDITABLE (sess->gui->input_box));
+		gtk_editable_set_position (GTK_EDITABLE (sess->gui->input_box), pos);
+	} else
+	{
+		/* we don't support changing non-front tabs yet */
+	}
+}
+
+void
+fe_set_inputbox_contents (session *sess, char *text)
+{
+	if (!sess->gui->is_tab || sess == current_tab)
+	{
+		gtk_entry_set_text (GTK_ENTRY (sess->gui->input_box), text);
+	} else
+	{
+		if (sess->res->topic_text)
+			free (sess->res->topic_text);
+		sess->res->topic_text = strdup (text);
+	}
+}
