@@ -1,7 +1,7 @@
 extern GtkStyle *input_style;
 extern GtkWidget *parent_window;
 
-void mg_changui_new (session *sess, restore_gui *res, int tab);
+void mg_changui_new (session *sess, restore_gui *res, int tab, int focus);
 void mg_update_xtext (GtkWidget *wid);
 void mg_safe_quit (void);
 void mg_switch_page (int relative, int num);
