@@ -844,7 +844,7 @@ menu_newserver_window (GtkWidget * wid, gpointer none)
 	int old = prefs.tabchannels;
 
 	prefs.tabchannels = 0;
-	new_ircwindow (NULL, NULL, SESS_SERVER);
+	new_ircwindow (NULL, NULL, SESS_SERVER, 0);
 	prefs.tabchannels = old;
 }
 
@@ -854,7 +854,7 @@ menu_newchannel_window (GtkWidget * wid, gpointer none)
 	int old = prefs.tabchannels;
 
 	prefs.tabchannels = 0;
-	new_ircwindow (current_sess->server, NULL, SESS_CHANNEL);
+	new_ircwindow (current_sess->server, NULL, SESS_CHANNEL, 0);
 	prefs.tabchannels = old;
 }
 
@@ -862,10 +862,15 @@ static void
 menu_newserver_tab (GtkWidget * wid, gpointer none)
 {
 	int old = prefs.tabchannels;
+	int oldf = prefs.newtabstofront;
 
 	prefs.tabchannels = 1;
-	new_ircwindow (NULL, NULL, SESS_SERVER);
+	/* force focus if setting is "only requested tabs" */
+	if (prefs.newtabstofront == 2)
+		prefs.newtabstofront = 1;
+	new_ircwindow (NULL, NULL, SESS_SERVER, 0);
 	prefs.tabchannels = old;
+	prefs.newtabstofront = oldf;
 }
 
 static void
@@ -874,7 +879,7 @@ menu_newchannel_tab (GtkWidget * wid, gpointer none)
 	int old = prefs.tabchannels;
 
 	prefs.tabchannels = 1;
-	new_ircwindow (current_sess->server, NULL, SESS_CHANNEL);
+	new_ircwindow (current_sess->server, NULL, SESS_CHANNEL, 0);
 	prefs.tabchannels = old;
 }
 
@@ -920,6 +925,12 @@ menu_search ()
 	search_open (current_sess);
 }
 
+static void
+menu_resetmarker (GtkWidget * wid, gpointer none)
+{
+	gtk_xtext_reset_marker_pos (GTK_XTEXT (current_sess->gui->xtext));
+}
+
 static void
 menu_flushbuffer (GtkWidget * wid, gpointer none)
 {
@@ -963,7 +974,7 @@ menu_wallops (GtkWidget * wid, gpointer none)
 			mode[0] = '+';
 		else
 			mode[0] = '-';
-		serv->p_nick_mode (serv, serv->nick, mode);
+		serv->p_mode (serv, serv->nick, mode);
 	}
 }
 
@@ -981,7 +992,7 @@ menu_servernotice (GtkWidget * wid, gpointer none)
 			mode[0] = '+';
 		else
 			mode[0] = '-';
-		serv->p_nick_mode (serv, serv->nick, mode);
+		serv->p_mode (serv, serv->nick, mode);
 	}
 }
 
@@ -1005,7 +1016,7 @@ menu_invisible (GtkWidget * wid, gpointer none)
 			mode[0] = '+';
 		else
 			mode[0] = '-';
-		serv->p_nick_mode (serv, serv->nick, mode);
+		serv->p_mode (serv, serv->nick, mode);
 	}
 }
 
@@ -1296,11 +1307,11 @@ static struct mymenu mymenu[] = {
 	{N_("S_ettings"), 0, 0, M_NEWMENU, 0, 1},	/* 33 */
 	{N_("Preferences..."), menu_settings, GTK_STOCK_PREFERENCES, M_MENUSTOCK, 0, 1},
 
-	{N_("Lists"), 0, GTK_STOCK_JUSTIFY_LEFT, M_MENUSUB, 0, 1},
+	{N_("Advanced"), 0, GTK_STOCK_JUSTIFY_LEFT, M_MENUSUB, 0, 1},
 	{N_("Auto Replace..."), menu_rpopup, 0, M_MENU, 0, 1},
 	{N_("CTCP Replies..."), menu_ctcpguiopen, 0, M_MENU, 0, 1},
 	{N_("Dialog Buttons..."), menu_dlgbuttons, 0, M_MENU, 0, 1},
-	{N_("Key Bindings..."), menu_keypopup, 0, M_MENU, 0, 1},
+	{N_("Keyboard Shortcuts..."), menu_keypopup, 0, M_MENU, 0, 1},
 	{N_("Text Events..."), menu_evtpopup, 0, M_MENU, 0, 1},
 	{N_("URL Handlers..."), menu_urlhandlers, 0, M_MENU, 0, 1},
 	{N_("User Commands..."), menu_usercommands, 0, M_MENU, 0, 1},
@@ -1329,13 +1340,18 @@ static struct mymenu mymenu[] = {
 	{N_("Raw Log..."), menu_rawlog, 0, M_MENU, 0, 1},	/* 51 */
 	{N_("URL Grabber..."), url_opengui, 0, M_MENU, 0, 1},
 	{0, 0, 0, M_SEP, 0, 0},
+	{N_("Reset Marker Line"), menu_resetmarker, 0, M_MENU, 0, 1, GDK_m},
 	{N_("C_lear Text"), menu_flushbuffer, GTK_STOCK_CLEAR, M_MENUSTOCK, 0, 1, GDK_l},
 	{N_("Search Text..."), menu_search, GTK_STOCK_FIND, M_MENUSTOCK, 0, 1, GDK_f},
 	{N_("Save Text..."), menu_savebuffer, GTK_STOCK_SAVE, M_MENUSTOCK, 0, 1},
 
 	{N_("_Help"), 0, 0, M_NEWMENU, 0, 1},	/* 62 */
 	{N_("_Contents"), menu_docs, GTK_STOCK_HELP, M_MENUSTOCK, 0, 1, GDK_F1},
+#ifdef GTK_STOCK_ABOUT
+	{N_("_About"), menu_about, GTK_STOCK_ABOUT, M_MENUSTOCK, 0, 1},
+#else
 	{N_("_About"), menu_about, (char *)&pix_about, M_MENUPIX, 0, 1},
+#endif
 
 	{0, 0, 0, M_END, 0, 0},
 };
@@ -1356,6 +1372,19 @@ create_icon_menu (char *labeltext, void *stock_name, int is_stock)
 	return item;
 }
 
+#if GTK_CHECK_VERSION(2,4,0)
+
+/* Override the default GTK2.4 handler, which would make menu
+   bindings not work when the menu-bar is hidden. */
+static gboolean
+menu_canacaccel (GtkWidget *widget, guint signal_id, gpointer user_data)
+{
+	/* GTK2.2 behaviour */
+	return GTK_WIDGET_IS_SENSITIVE (widget);
+}
+
+#endif
+
 GtkWidget *
 menu_create_main (void *accel_group, int bar, int away, int toplevel,
 						GtkWidget **away_item_ret, GtkWidget **user_menu_ret)
@@ -1376,6 +1405,11 @@ menu_create_main (void *accel_group, int bar, int away, int toplevel,
 	else
 		menu_bar = gtk_menu_new ();
 
+#if GTK_CHECK_VERSION(2,4,0)
+	g_signal_connect (G_OBJECT (menu_bar), "can-activate-accel",
+							G_CALLBACK (menu_canacaccel), 0);
+#endif
+
 	mymenu[19-menuoffset].state = prefs.invisible;
 	mymenu[20-menuoffset].state = prefs.wallops;
 	mymenu[21-menuoffset].state = prefs.servernotice;
